//
//  WGBScrollContainerView.m
//  跑马灯
//
//  Created by Wangguibin on 2017/8/22.
//  Copyright © 2017年 王贵彬. All rights reserved.
//

#import "WGBScrollContainerView.h"

@interface WGBScrollContainerView ()<UIScrollViewDelegate>

@property (nonatomic,strong) UIScrollView *bgScrollView;
@property (nonatomic,strong) NSTimer *timer;
@property (nonatomic,assign) NSInteger flagIndex;
@property (nonatomic,copy) void(^clickIndex) (NSInteger index);

@property (nonatomic,assign,readwrite) NSTimeInterval duration;
@property (nonatomic,assign,readwrite) WGBScrollDirectionType directionType;

@end

@implementation WGBScrollContainerView

- (UIScrollView *)bgScrollView{
	if (!_bgScrollView) {
		_bgScrollView = [[UIScrollView alloc] initWithFrame:self.bounds];
		_bgScrollView.delegate = self;
		_bgScrollView.backgroundColor = [UIColor redColor];
		_bgScrollView.showsVerticalScrollIndicator = NO;
		_bgScrollView.showsHorizontalScrollIndicator = NO;
		_bgScrollView.bounces = NO;
		[self addSubview: _bgScrollView];
	}
	return _bgScrollView;
}

- (instancetype)initWithFrame:(CGRect)frame
{
	self = [super initWithFrame:frame];
	if (self) {

	}
	return self;
}

#pragma mark - 即将进入窗口
- (void)willMoveToWindow:(UIWindow *)newWindow
{
	[super willMoveToWindow:newWindow];
	[self reload];
}

- (void)reload{
	for (UIView *view in self.subviews) {
		[view removeFromSuperview];
	}

	[self stop];

		///没有设置数据源 return
	if (self.wgbDataSourceDalegate == nil) {
		return;
	}

	if (![self.wgbDataSourceDalegate respondsToSelector:@selector(wgb_autoScrollDuration)]) {
		@throw [NSException exceptionWithName:@"WGBError" reason:@"未实现（wgb_autoScrollDuration:）" userInfo:nil];
	}

	if (![self.wgbDataSourceDalegate respondsToSelector:@selector(wgb_ScrollDirectionType)]) {
		@throw [NSException exceptionWithName:@"WGBError" reason:@"未实现（wgb_ScrollDirectionType:）" userInfo:nil];
	}

	if (![self.wgbDataSourceDalegate respondsToSelector:@selector(wgb_numberOfRowsInWithContainerView:)]) {
		@throw [NSException exceptionWithName:@"WGBError" reason:@"未实现（wgb_numberOfRowsInWithContainerView:）" userInfo:nil];
	}
	
	if (![self.wgbDataSourceDalegate respondsToSelector:@selector(wgb_subContentViewWithContainerView:subViewForRowAtIndex:)]) {
		@throw [NSException exceptionWithName:@"WGBError" reason:@"未实现（wgb_subContentViewWithContainerView:subViewForRowAtIndex:）" userInfo:nil];
	}

	self.duration = [self.wgbDataSourceDalegate wgb_autoScrollDuration];
	self.directionType = [self.wgbDataSourceDalegate wgb_ScrollDirectionType];

	[self setup];
	[self start];
}


- (void)setDirectionType:(WGBScrollDirectionType)directionType{
	_directionType = directionType;

}

- (void)setDuration:(NSTimeInterval)duration{
	_duration = duration;
	self.timer = [NSTimer scheduledTimerWithTimeInterval: duration target:self selector:@selector(changeContent) userInfo:nil repeats:YES];
	[[NSRunLoop mainRunLoop] addTimer: self.timer forMode:NSRunLoopCommonModes];
	self.flagIndex = 0;
}


- (void)setup{
	CGRect frame = self.bounds;
	CGFloat width = frame.size.width;
	CGFloat height = frame.size.height;
	CGFloat count = [self.wgbDataSourceDalegate wgb_numberOfRowsInWithContainerView: self];

	for (NSInteger i = 0; i < count ; i += 1) {
		UIView *subView = [self.wgbDataSourceDalegate wgb_subContentViewWithContainerView:self subViewForRowAtIndex: i];
		if (self.directionType == WGBScrollDirectionTypeVertical) {
			subView.frame = CGRectMake(0, height*i , width , height);
			self.bgScrollView.contentSize = CGSizeMake(width, height*count);
		}else{
			subView.frame = CGRectMake(width*i, 0, width , height);
			self.bgScrollView.contentSize = CGSizeMake(width*count, height);
		}
		subView.tag = i;
		subView.userInteractionEnabled = YES;
		UITapGestureRecognizer *tapGes = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(clickItemViewIndex:)];
		[subView addGestureRecognizer: tapGes];
		[self.bgScrollView addSubview: subView];
	}

	[self.bgScrollView setContentOffset:CGPointMake(0, 0)];
}

- (void)clickItemViewIndex:(UITapGestureRecognizer *)tap{
	if (self.clickIndex) {
		self.clickIndex(tap.view.tag);
	}

	if ([self.wgbDataSourceDalegate respondsToSelector:@selector(wgb_containerView:didSelectRowAtIndex:)]) {
		[self.wgbDataSourceDalegate wgb_containerView:self didSelectRowAtIndex:tap.view.tag];
	}
}


- (void)clickItemViewWithIndex:(void (^)(NSInteger index))clickBlock{
	self.clickIndex = clickBlock;
}


- (void)changeContent{
	CGRect frame = self.bounds;
	CGFloat width = frame.size.width;
	CGFloat height = frame.size.height;
	CGFloat count =  [self.wgbDataSourceDalegate wgb_numberOfRowsInWithContainerView: self];

	if (self.flagIndex == count) {
		self.flagIndex = 0;
		[self.bgScrollView setContentOffset:CGPointMake(0, 0)];
	}

	if (self.directionType == WGBScrollDirectionTypeVertical) {
		[self.bgScrollView setContentOffset:CGPointMake(0, height*self.flagIndex) animated:YES];
	}else{
		[self.bgScrollView setContentOffset:CGPointMake(width *self.flagIndex, 0) animated:YES];
	}
	self.flagIndex++;
}

- (void)start{
	[self.timer fire];
}

- (void)stop{
	[self.timer invalidate];
	self.timer = nil;
}

- (void)pause{
	[self.timer setFireDate:[NSDate distantPast]];
}

- (void)dealloc{
	[self stop];
}


@end
